#pragma once
#include "LSFramework.Exports.h"
#include "LSMacros.h"
#include <string>

namespace LSFramework
{
  class LSFRAMEWORK_API CString
  {
  public:
    // - Constructor
    CString() = default;

    CString(const char* pData);

    CString(const CString& sString);

    CString(const std::string& sString);

    // - Operator
    CString& operator=(const CString& sRhs);

    // - Destructor
    ~CString();

    // - Alloc
#pragma region Conversion
    inline void Assign(const CString& sString)
    {
      Assign(sString.ToChar());
    }
    inline void Assign(const std::string& sString)
    {
      Assign(sString.c_str());
    }
    //-----------------------------------------------------------------------------------------------------------------------------
    inline void Append(const CString& sRhs)
    {
      Insert(sRhs, sRhs.Size());
    }
    inline void Append(const std::string& sRhs)
    {
      Insert(sRhs, sRhs.length());
    }
    inline void Append(const char* pData)
    {
      Insert(pData, StringLength(pData), m_zSize);
    }
    //-----------------------------------------------------------------------------------------------------------------------------
    inline void Insert(const CString& sRhs, size_t zPos)
    {
      Insert(sRhs.m_pData, sRhs.Size(), zPos);
    }
    inline void Insert(const std::string& sRhs, size_t zPos)
    {
      Insert(sRhs.c_str(), sRhs.size(), zPos);
    }
    //-----------------------------------------------------------------------------------------------------------------------------
    inline bool Remove(const CString& sRhs)
    {
      return RemoveAt(Find(sRhs), sRhs.m_zSize);
    }
    inline bool Remove(const std::string& sRhs)
    {
      return RemoveAt(Find(sRhs), sRhs.length());
    }
    inline bool Remove(const char* pData)
    {
      return RemoveAt(Find(pData), StringLength(pData));
    }
#pragma endregion

#pragma region Logic
    void Assign(const char* pData);
    void Insert(const char* pData, size_t zLength, size_t zPos);
    bool RemoveAt(size_t uiPos, size_t uiSize);
#pragma endregion

    // - Utility
#pragma region Conversion
    inline bool Contains(const CString& sRhs) const
    {
      return Find(sRhs) == 0;
    }
    inline bool Contains(const std::string& sRhs) const
    {
      return Find(sRhs) == 0;
    }

    inline bool Contains(const char* pData) const
    {
      return Find(pData) == 0;
    }

    //-----------------------------------------------------------------------------------------------------------------------------
    inline int Find(const CString& sRhs, size_t uiOffset = 0) const
    {
      return Find(sRhs.m_pData, uiOffset);
    }
    inline int Find(const std::string& sRhs, size_t uiOffset = 0) const
    {
      return Find(sRhs.c_str(), uiOffset);
    }

    inline int Find(const char* pData, size_t uiOffset = 0) const
    {
      return Find(pData, uiOffset, StringLength(pData));
    }

    inline int Find(const CString& sRhs, size_t uiOffset, size_t uiLength) const
    {
      return Find(sRhs.m_pData, uiOffset, uiLength);
    }
    inline int Find(const std::string& sRhs, size_t uiOffset, size_t uiLength) const
    {
      return Find(sRhs.c_str(), uiOffset, uiLength);
    }
#pragma endregion

#pragma region Logic
    size_t StringLength(const char* pData) const;

    int Find(const char* pData, size_t uiOffset, size_t uiLength) const;

    //-----------------------------------------------------------------------------------------------------------------------------
    inline char At(size_t uiPos) const
    {
      return *(m_pData + uiPos);
    }
    //-----------------------------------------------------------------------------------------------------------------------------
    inline size_t Size() const
    {
      return m_zSize;
    }
    //-----------------------------------------------------------------------------------------------------------------------------
    inline size_t Capacity() const
    {
      return m_uiCapacity;
    }
    //-----------------------------------------------------------------------------------------------------------------------------
    inline std::string ToStdString() const
    {
      return std::string(m_pData);
    }
    //-----------------------------------------------------------------------------------------------------------------------------
    inline const char* ToChar() const
    {
      return m_pData;
    }
#pragma endregion

  private:
    void IncreaseCapacity(size_t uiNewLength = 0);

    char* m_pData = nullptr;
    size_t m_zSize = 0;
    size_t m_uiCapacity = 0;
    const float c_uiCapacityIncreaseFactor = 1.5;
  };

  //-----------------------------------------------------------------------------------------------------------------------------
  inline CString operator+(const CString& sLhs, const CString& sRhs)
  {
    CString sNew(sLhs);
    sNew.Append(sRhs);
    return sNew;
  }
  inline CString operator+(const CString& sLhs, const char* sRhs)
  {
    CString sNew(sLhs);
    sNew.Append(sRhs);
    return sNew;
  }
  inline CString operator+(const char* sLhs, const CString& sRhs)
  {
    CString sNew(sLhs);
    sNew.Append(sRhs);
    return sNew;
  }
}